"use strict";

// TODO: apn does not support the new HTTP/2 protocal. It is fine to use it in V1,
// but probably we will replace it in the future.

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _apn = require('apn');

var _apn2 = _interopRequireDefault(_apn);

var _parse = require('parse');

var _parse2 = _interopRequireDefault(_parse);

var _npmlog = require('npmlog');

var _npmlog2 = _interopRequireDefault(_npmlog);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var LOG_PREFIX = 'parse-server-push-adapter ApnsToken';

/**
 * Create a new connection to the APN service.
 * @constructor
 * @param {Object|Array} args An argument or a list of arguments to config APNS connection
 * @param {String} args.cert The filename of the connection certificate to load from disk
 * @param {String} args.key The filename of the connection key to load from disk
 * @param {String} args.pfx The filename for private key, certificate and CA certs in PFX or PKCS12 format, it will overwrite cert and key
 * @param {String} args.passphrase The passphrase for the connection key, if required
 * @param {String} args.bundleId The bundleId for cert
 * @param {Boolean} args.production Specifies which environment to connect to: Production (if true) or Sandbox
 * @param {String} args.tokenKey The Path of the apns key which is in p8 format
 * @param {String} args.tokenKeyId  The Key ID of the p8 file (available at https://developer.apple.com/account/ios/certificate/key)
 * @param {String} args.tokenTeamId The Team ID of your Apple Developer Account (available at https://developer.apple.com/account/#/membership/)
 */
var apnProvider;

function ApnsToken(args) {

  if (args.tokenKey && args.tokenKeyId && args.tokenTeamId) {
    apnProvider = new _apn2.default.Provider({
      token: {
        key: args.tokenKey,
        keyId: args.tokenKeyId,
        teamId: args.tokenTeamId
      },
      production: args.production
    });
    return;
  }

  apnProvider = new _apn2.default.Provider({
    cert: args.cert,
    key: args.key,
    pfx: args.pfx,
    passphrase: args.passphrase,
    bundleId: args.bundleId,
    production: args.production
  });
}

/**
 * Send apns request.
 * @param {Object} data The data we need to send, the format is the same with api request body
 * @param {Array} devices A array of devices
 * @returns {Object} A promise which is resolved immediately
 */
ApnsToken.prototype.send = function (data, devices) {
  var coreData = data.data;
  var expirationTime = data['expiration_time'];
  var notification = generateNotification(coreData, expirationTime);
  var allPromises = [];
  devices.forEach(function (device) {
    notification.topic = device.appIdentifier;
    apnProvider.send(notification, device.deviceToken).then(function (result) {
      var promise = Promise.resolve({
        transmitted: true,
        device: {
          deviceToken: device.deviceToken,
          deviceType: 'ios'
        },
        result: result
      });
      allPromises.push(promise);
    }, function (error) {
      var promise = Promise.resolve({
        transmitted: false,
        device: {
          deviceToken: device.deviceToken,
          deviceType: 'ios'
        },
        result: { error: error }
      });
      allPromises.push(promise);
    });
  });
  return Promise.all(allPromises);
};

/**
 * Generate the apns notification from the data we get from api request.
 * @param {Object} coreData The data field under api request body
 * @param {number} expirationTime The expiration time in milliseconds since Jan 1 1970
 * @returns {Object} A apns notification
 */
function generateNotification(coreData, expirationTime) {
  var notification = new _apn2.default.Notification();
  var payload = {};
  for (var key in coreData) {
    switch (key) {
      case 'alert':
        notification.alert = coreData.alert;
        break;
      case 'badge':
        notification.badge = coreData.badge;
        break;
      case 'sound':
        notification.sound = coreData.sound;
        break;
      case 'content-available':
        notification.setNewsstandAvailable(true);
        var isAvailable = coreData['content-available'] === 1;
        notification.setContentAvailable(isAvailable);
        break;
      case 'mutable-content':
        var isMutable = coreData['mutable-content'] === 1;
        notification.setMutableContent(isMutable);
        break;
      case 'category':
        notification.category = coreData.category;
        break;
      default:
        payload[key] = coreData[key];
        break;
    }
  }
  notification.payload = payload;
  notification.expiry = expirationTime / 1000;
  return notification;
}

ApnsToken.generateNotification = generateNotification;

if (process.env.TESTING) {}
module.exports = ApnsToken;
exports.default = ApnsToken;